@@ -84,7 +84,7 @@ print_dump_header (gzFile output, int output_format, int include)
 	sep = ",";
     }
     if (include & DUMP_INCLUDE_TAGS) {
-	gzprintf (output, "%sproperties", sep);
+	gzprintf (output, "%stags", sep);
     }
     gzputs (output, "\n");
 }
@@ -250,7 +250,8 @@ database_dump_file (notmuch_database_t *notmuch, gzFile output,
 
 	message = notmuch_messages_get (messages);
 
-	if (dump_tags_message (notmuch, message, output_format, output,
+	if ((include & DUMP_INCLUDE_TAGS) &&
+	    dump_tags_message (notmuch, message, output_format, output,
 			       &buffer, &buffer_size))
 	    return EXIT_FAILURE;
 
