@@ -2,6 +2,13 @@
 test_description="\"notmuch dump\" and \"notmuch restore\""
 . ./test-lib.sh || exit 1
 
+NOTMUCH_NEW > /dev/null
+test_begin_subtest "dump header"
+cat <<EOF > EXPECTED
+#notmuch-dump batch-tag:3 config,properties,tags
+EOF
+notmuch dump > OUTPUT
+test_expect_equal_file EXPECTED OUTPUT
 add_email_corpus
 
 test_expect_success 'Dumping all tags' \
