@@ -41,7 +41,7 @@ output=$(notmuch show --format=json --verify subject:"test signed message 001" \
 expected='[[[{"id": "XXXXX",
  "match": true,
  "excluded": false,
- "filename": "YYYYY",
+ "filename": ["YYYYY"],
  "timestamp": 946728000,
  "date_relative": "2000-01-01",
  "tags": ["inbox","signed"],
@@ -75,7 +75,7 @@ output=$(notmuch show --format=json --verify subject:"test signed message 001" \
 expected='[[[{"id": "XXXXX",
  "match": true,
  "excluded": false,
- "filename": "YYYYY",
+ "filename": ["YYYYY"],
  "timestamp": 946728000,
  "date_relative": "2000-01-01",
  "tags": ["inbox","signed"],
@@ -109,7 +109,7 @@ output=$(notmuch show --format=json --verify subject:"test signed message 001" \
 expected='[[[{"id": "XXXXX",
  "match": true,
  "excluded": false,
- "filename": "YYYYY",
+ "filename": ["YYYYY"],
  "timestamp": 946728000,
  "date_relative": "2000-01-01",
  "tags": ["inbox","signed"],
@@ -183,7 +183,7 @@ output=$(notmuch show --format=json --decrypt subject:"test encrypted message 00
 expected='[[[{"id": "XXXXX",
  "match": true,
  "excluded": false,
- "filename": "YYYYY",
+ "filename": ["YYYYY"],
  "timestamp": 946728000,
  "date_relative": "2000-01-01",
  "tags": ["encrypted","inbox"],
@@ -205,6 +205,7 @@ expected='[[[{"id": "XXXXX",
  "content": "This is a test encrypted message.\n"},
  {"id": 5,
  "content-type": "application/octet-stream",
+ "content-disposition": "attachment",
  "content-length": "NONZERO",
  "content-transfer-encoding": "base64",
  "filename": "TESTATTACHMENT"}]}]}]},
@@ -240,7 +241,7 @@ output=$(notmuch show --format=json --decrypt subject:"test encrypted message 00
 expected='[[[{"id": "XXXXX",
  "match": true,
  "excluded": false,
- "filename": "YYYYY",
+ "filename": ["YYYYY"],
  "timestamp": 946728000,
  "date_relative": "2000-01-01",
  "tags": ["encrypted","inbox"],
@@ -276,7 +277,7 @@ output=$(notmuch show --format=json --decrypt subject:"test encrypted message 00
 expected='[[[{"id": "XXXXX",
  "match": true,
  "excluded": false,
- "filename": "YYYYY",
+ "filename": ["YYYYY"],
  "timestamp": 946728000,
  "date_relative": "2000-01-01",
  "tags": ["encrypted","inbox"],
@@ -350,7 +351,7 @@ output=$(notmuch show --format=json --verify subject:"test signed message 001" \
 expected='[[[{"id": "XXXXX",
  "match": true,
  "excluded": false,
- "filename": "YYYYY",
+ "filename": ["YYYYY"],
  "timestamp": 946728000,
  "date_relative": "2000-01-01",
  "tags": ["inbox","signed"],
